/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.model;

import nl.cloudfarming.client.model.EnterpriseIdProvider;
import nl.cloudfarming.client.model.ItemIdType;

/**
 *
 * @author Maciek Dulkiewicz && Wytse Visser
 */
public class Implement extends Machine {

    private transient ImplementGeographical implementGeographical;
    public static final String EXT = "imt";
    public static final ItemIdType ITEM_ID_TYPE = ItemIdType.IMT;
    private double workWidth = 0.0d;
    public static final String PROP_WORKWIDTH = "workWidth";

    public Implement(EnterpriseIdProvider eip, String name) {
        super(eip, ITEM_ID_TYPE, name);
    }

    public double getWorkWidth() {
        return workWidth;
    }

    public void setWorkWidth(double workWidth) {
        double old = this.workWidth;
        this.workWidth = workWidth;
        getPropertyChangeSupport().firePropertyChange(PROP_WORKWIDTH, old, workWidth);
    }

    @Override
    public MachineGeometrical getGeometrical() {
        if (implementGeographical == null) {
            implementGeographical = new ImplementGeographical();
        }
        return implementGeographical;
    }

    private class ImplementGeographical extends MachineGeometrical {

        public ImplementGeographical() {
            super(Implement.this);
        }

        @Override
        public String getIconPath() {
            return "nl/cloudfarming/client/fleet/implement/widgets32.png";
        }
    }
}
